/*
        FILE :                  camera.cpp

        DESCRIPTION :           Camera routines

        AUTHOR :                Trax / Aspirine (c), 1999

        VERSION :               0.1b

*/

//// INCLUDES /////////////////////////////////////////////////////////////////

#include "camera.h"
#include "matrix.h"
#include <math.h>

//// FUNCTIONS ////////////////////////////////////////////////////////////////

void init_camera(CAMERA *camera, float PERSPECTIVE, float CENTER_X,
                 float CENTER_Y, float x_suhde, float y_suhde,
                 int screen_left, int screen_right,
                 int screen_up, int screen_down)
{
        camera->perspective = PERSPECTIVE;
        camera->center_x = CENTER_X;
        camera->center_y = CENTER_Y;
        camera->x_suhde = x_suhde;
        camera->y_suhde = y_suhde;
        camera->screen_left = screen_left;
        camera->screen_right = screen_right;
        camera->screen_up = screen_up;
        camera->screen_down = screen_down;

        matrix_normalize(camera->cam); // prepare camera matrix
        camera->xpos = 0;   
        camera->ypos = 0;
        camera->zpos = -600;
}

///////////////////////////////////////////////////////////////////////////////

void camera_update(CAMERA *camera, float xa, float ya, float za,
                   float s_rt, float s_dn, float s_fw)
{
        MATRIX chg;

        // prepare camera rotation matrix
        matrix_update(chg,sin(xa), sin(ya), sin(za),
                        cos(xa), cos(ya), cos(za));

        matrix_mul(camera->cam, chg); // rotate camera

        // move camera:
        camera->xpos += s_rt * camera->cam[0][0] +
                        s_dn * camera->cam[0][1] +
                        s_fw * camera->cam[0][2];
        camera->ypos += s_rt * camera->cam[1][0] +
                        s_dn * camera->cam[1][1] +
                        s_fw * camera->cam[1][2];
        camera->zpos += s_rt * camera->cam[2][0] +
                        s_dn * camera->cam[2][1] +
                        s_fw * camera->cam[2][2];
}



