/*
        FILE :                  light.cpp

        DESCRIPTION :           Implementation of light routines

        AUTHOR :                Trax / Aspirine (c), 1999

        VERSION :               0.1b

	NOTE : inline ?		
*/

//// INCLUDES /////////////////////////////////////////////////////////////////

#include "light.h"
#include "vector.h"
#include "matrix.h"
#include <math.h>

//// FUNCTIONS ////////////////////////////////////////////////////////////////

/*inline*/ int light_intensity(VERTEX vertexnormal, LIGHT light, int scale)
{
        scale *= (light.direction.x * vertexnormal.x +  
                 light.direction.y * vertexnormal.y +  
                 light.direction.z * vertexnormal.z);  
        if (scale < 1) scale = 1;
        return scale;
}

///////////////////////////////////////////////////////////////////////////////

inline float get_flat_inten(POL_VERTEX v1, POL_VERTEX v2,
                            POL_VERTEX v3, LIGHT light)
{
        float ox, oy, oz, len;

        calcnormal(v1.x, v1.y, v1.z, v2.x, v2.y, v2.z, v3.x, v3.y, v3.z, &ox, &oy, &oz);
        len = sqrt(ox * ox + oy * oy + oz * oz);
        if (len == 0.0) len = -0.1;
        ox = (light.direction.x * ox +
             light.direction.y * oy +
             light.direction.z * oz) / len;
        if (ox < 0) ox = 0;
        return ox;
}

///////////////////////////////////////////////////////////////////////////////

void light_set(LIGHT *light, float x, float y, float z)
{
        float len = sqrt(x * x + y * y + z * z);
        light->direction.x = x / len;
        light->direction.y = y / len;
        light->direction.z = z / len;
}

///////////////////////////////////////////////////////////////////////////////

void light_update(LIGHT *light, float xa, float ya, float za)
{
        MATRIX rota;

        // prepare light rotation matrix
        matrix_update(rota,
                     sin(xa), sin(ya), sin(za),
                     cos(xa), cos(ya), cos(za));

        matrix_mul_vector(light->direction, &light->direction, rota); 
}


